unit MAIN;

interface

uses Windows, ShellAPI, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, StdActns,
  IdHTTPHeaderInfo, ActnList, ToolWin, ImgList, IniFiles,utils,pagefound,splashscreen;

{generic functions}
procedure validateKeywords(var txtKeywords: TMemo);
procedure processMessagesAtInterval;

const
   WM_NOTIFYICON  = WM_USER+333;
   MIN_REFRESH_VALUE = 60;

   SOCKET_TIMEOUT = 60000;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Window1: TMenuItem;
    FileExitItem: TMenuItem;
    WindowCascadeItem: TMenuItem;
    WindowTileItem: TMenuItem;
    WindowArrangeItem: TMenuItem;
    WindowMinimizeItem: TMenuItem;
    StatusBar: TStatusBar;
    ActionList1: TActionList;
    EditCut1: TEditCut;
    EditCopy1: TEditCopy;
    EditPaste1: TEditPaste;
    FileNew1: TAction;
    FileSave1: TAction;
    FileExit1: TAction;
    FileOpen1: TAction;
    FileSaveAs1: TAction;
    WindowCascade1: TWindowCascade;
    WindowTileHorizontal1: TWindowTileHorizontal;
    WindowArrangeAll1: TWindowArrange;
    WindowMinimizeAll1: TWindowMinimizeAll;
    HelpAbout1: TAction;
    FileClose1: TWindowClose;
    WindowTileVertical1: TWindowTileVertical;
    WindowTileItem2: TMenuItem;
    ToolBar2: TToolBar;
    ToolButton8: TToolButton;
    ToolButton10: TToolButton;
    ImageList1: TImageList;
    RSSfeeds1: TMenuItem;
    ToolButton1: TToolButton;
    sbRSS: TSpeedButton;
    sbSaveConfig: TSpeedButton;
    SpeedButton1: TSpeedButton;
    btnNewWebPage: TSpeedButton;
    btnSpider: TSpeedButton;
    Newwebsearchwindow1: TMenuItem;
    Newwebpagemonitorwindow1: TMenuItem;
    procedure FileExit1Execute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure RSSfeeds1Click(Sender: TObject);
    procedure sbSaveConfigClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure btnNewWebPageClick(Sender: TObject);
    procedure btnSpiderClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    TrayIcon: TNotifyIconData;
    HMainIcon: HICON;
    procedure CreateMDIChild;
  public
    { Public declarations }
    proxyDetails: TIdProxyConnectionInfo;
    procedure initialise;
    procedure saveConfig;
    procedure ClickTrayIcon(var msg: TMessage); message WM_NOTIFYICON;
    procedure MinimizeClick(Sender:TObject);
    function alertUser(formCaption,title,messageText: string; url: string; showRelevancePanel: boolean): TUserResponse;
    procedure bringApplicationToFront;
  end;

const
   confFileName = 'webu.conf';
   appVersion = '1.19';
   MINUTE = 60000;

var
  MainForm: TMainForm;
  lastProcessMessages: COMP;
  initialised, shownWelcome, firstTime, feedsFormShowing: boolean;
  repeatInterval,runAfterIdleFor,processMessagesInterval: integer;
  verboseOutput,forceToForeground,showNagScreens,backupDataFiles: boolean;



implementation

{$R *.dfm}

uses pagemonitor,spider,feeds;

procedure TMainForm.CreateMDIChild;
var
  frmPageMonitor: TfrmPageMonitor;
begin
  frmPageMonitor := TfrmPageMonitor.Create(Application);
  frmPageMonitor.Caption := 'Window ' + IntToStr(MDIChildCount-1);;
  frmPageMonitor.setOwner(self);
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
   if (FirstTime) and (not shownWelcome) then
   begin
      ShowMessage('To set up RSS/Atom feed monitoring click the orange icon.'+chr(13)+chr(10) +
      'To monitor a standard web site, click the web icon (multiple windows can be opened)');

      shownWelcome := true;
   end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   frmFeeds.cancelled := true;
   frmFeeds.Timer1.Enabled := false;
   saveConfig;
end;

procedure TMainForm.saveConfig;
var
   i,browserWindowCount: integer;
   confFile: TIniFile;
   fullPathConfFile: string;
begin
   Screen.Cursor := crHourglass;
   fullPathConfFile := ExtractFilePath(Application.ExeName) + '\' + confFileName;
   if backupDataFiles then
   begin
      deleteFile(fullPathConfFile + '.bak');
      copyFile(pchar(fullPathConfFile),pchar(fullPathConfFile + '.bak'),true);
   end;

   confFile := TIniFile.Create(fullPathConfFile);
   try
      confFile.WriteString('main','appVersion',appVersion);
      confFile.WriteString('main','firstTime','false');
      browserWindowCount := 0;
      for i:= 0 to MdiChildCount - 1 do
      begin
         if MDIChildren[i].ClassType = TfrmPageMonitor then
         begin
            (MDIChildren[i] as TfrmPageMonitor).saveConfig('window_' + IntToStr(browserWindowCount), confFile);
            inc(browserWindowCount);
         end;
      end;
      frmFeeds.saveConfig(confFile);
      frmFeeds.saveData;

      confFile.WriteInteger('main','browserWindowCount',browserWindowCount);
   finally
      confFile.free;
      Screen.Cursor := crDefault;
   end;

end;

procedure TMainForm.sbSaveConfigClick(Sender: TObject);
begin
   saveConfig;
   showmessage('Config saved');
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
   ShellExecute(GetDesktopWindow, 'open', PChar('http://members.lycos.co.uk/wuul/webu/readme.html'), nil, nil, SW_ShowNormal);
end;

procedure TMainForm.btnSpiderClick(Sender: TObject);
var
  spider: TfrmSpider;
begin
   if frmFeeds.getRelevantWordCount < 100 then
   begin
      ShowMessage('I have not yet gathered sufficient information about your interests.  Please run the learning process a few times on the Feeds page');
      exit;
   end;

   if frmFeeds.getRelevantWordCount < 1000 then
   begin
      if MessageDlg('I don''t really have sufficient information about your interests.  I recommend that you run the learning process a few more times on the Feeds page.'+chr(13)+chr(10)+chr(13)+chr(10)+
         'Do you wish to use this feature anyway?', mtConfirmation, [mbYes, mbNo], 0) <> mrYes then

         exit;
   end;

   feedsFormShowing := false;

   spider := TfrmSpider.Create(Application);
   spider.setOwner(self);
   spider.Caption := 'Window ' + IntToStr(MDIChildCount-1);

   if firstTime then ShowMessage('On this page you can search the web for relevant content, based on your prior feedback about the kind of articles you find interesting.'+chr(13)+chr(10)+chr(13)+chr(10)+
      'Enter a web site to start on, and I will follow the links on this site and search for content that you may be interested in.  '+
      'Adjust the slider to indicate how relevant a page needs to be for me to add it to the list.');
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
   initialised := false;
   shownWelcome := false;
   Application.HintHidePause := 30000; // how long hints should be shown for
   lastProcessMessages := 0;
   proxyDetails := TIdProxyConnectionInfo.Create;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
   proxyDetails.free;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
   // nasty hack to restore the feeds form if it was showing previously
   // for some reason it doesn't display when the main form is focused
   if feedsFormShowing then frmFeeds.BringToFront;
end;

procedure TMainForm.FileExit1Execute(Sender: TObject);
begin
   Close;
end;


procedure TMainForm.initialise;
var
   windowCount,i: integer;
   confFile: TIniFile;
   loadPagesOnStartup: boolean;
begin
   self.caption := self.caption + ' ' + appVersion;

   // system tray icon stuff
   HMainIcon:=LoadIcon(MainInstance, 'MAINICON');
   Shell_NotifyIcon(NIM_DELETE, @TrayIcon);
   with trayIcon do
   begin
      cbSize              := sizeof(TNotifyIconData);
      Wnd                 := handle;
      uID                 := 123;
      uFlags              := NIF_MESSAGE or NIF_ICON or NIF_TIP;
      uCallbackMessage    := WM_NOTIFYICON;
      hIcon               := HMainIcon;
      szTip               := 'webu'
   end;
   Application.OnMinimize:= MinimizeClick;

   // load windows
   confFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + '\' + confFileName);
   try

      // proxy config
      proxyDetails.ProxyServer := confFile.ReadString('main','proxyHost','');
      proxyDetails.ProxyPort := confFile.ReadInteger('main','proxyPort',0);
      proxyDetails.ProxyUsername := confFile.ReadString('main','proxyUsername','');
      proxyDetails.ProxyPassword := confFile.ReadString('main','proxyPassword','');
      proxyDetails.BasicAuthentication := confFile.readString('main','proxyBasicAuthentication','true') = 'true';

      // other config
      repeatInterval := confFile.readInteger('main','repeatInterval', 60) * MINUTE;
      loadPagesOnStartup := confFile.readString('main','loadPagesOnStartup','true') = 'true';
      verboseOutput := confFile.readString('main','verboseOutput','false') = 'true';
      forceToForeground := confFile.readString('main','forceToForeground','true') = 'true';
      showNagScreens := confFile.readString('main','showNagScreens','true') = 'true';
      runAfterIdleFor := confFile.readInteger('main','runAfterIdleFor',0) * 1000;
      processMessagesInterval := confFile.readInteger('main','processMessagesInterval',60) * 1000;
      firstTime := confFile.readString('main','firstTime','false') = 'true';
      backupDataFiles  := confFile.readString('main','backupDataFiles','true') = 'true';

      // load browser pages
      windowCount := confFile.readInteger('main','browserWindowCount',0);
      for i := 0 to windowCount-1 do
      begin
         CreateMDIChild;
         (MDIChildren[0] as TfrmPageMonitor).loadConfig('window_' + IntToStr(i), confFile);
         if loadPagesOnStartup then (MDIChildren[0] as TfrmPageMonitor).getWebPage;
      end;

      // RSS feed management form
      with frmFeeds do
      begin
         loadConfig(confFile);
         // if loadPagesOnStartup then searchAll(false); // removed as no need for it
      end;

      if confFile.readString('main','checkForNewVersion','true') = 'true' then
      begin
         frmSplashScreen.updateDetails('Checking for new version',0);
         try
            if checkForNewVersion('http://members.lycos.co.uk/wuul/webu/webu.xml', proxyDetails, StrToFloat(appVersion)) then
            begin
               if MessageDlg('A newer version of this program is available.  Do you want to visit the web site now so you can download the most recent version?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
                  ShellExecute(GetDesktopWindow, 'open', PChar('http://members.lycos.co.uk/wuul/webu/readme.html'), nil, nil, SW_ShowNormal)
            end;
         except
            ShowMessage('Unable to check for new version.' + chr(13)+chr(10) + chr(13)+chr(10) + 'The site may be down, or there could be a problem with your internet connection.');
         end;
      end;

   finally
      confFile.free;
   end;

   tile;
   initialised := true;

   frmSplashScreen.hide;
   // frmSplashScreen.free;
end;

procedure TMainForm.ClickTrayIcon(var msg: TMessage);
begin
  case msg.lparam of
    WM_LBUTTONUP, WM_LBUTTONDBLCLK :
    {WM_BUTTONDOWN may cause next Icon to activate if this icon is deleted -
        (Icons shift left and left neighbor will be under mouse at ButtonUp time)}
    begin
      Application.Restore;  {restore the application}
      If WindowState = wsMinimized then WindowState := wsNormal;  {Reset minimized state}
      {Added 5/6/04 ====>} visible:=true;
      SetForegroundWindow(Application.Handle); {Force form to the foreground }
      Shell_NotifyIcon(NIM_Delete, @TrayIcon);
    end;
  end;
end;

procedure TMainForm.MinimizeClick(Sender:TObject);
begin
   if not initialised then exit;
   Shell_NotifyIcon(NIM_Add, @TrayIcon);
   hide;
   {hide the taskbar button}
   if IsWindowVisible(Application.Handle) then
      ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TMainForm.bringApplicationToFront;
var
   currentFeedsFormShowing: boolean;
begin
   // don't force the feeds form to foreground
   // otherwise the entire application is shown
   // even if we have set forceToForeground to false
   currentFeedsFormShowing := feedsFormShowing;
   feedsFormShowing := false;

   beep;
   if isIconic(application.handle) then
   begin
      Application.Restore;
      If WindowState = wsMinimized then WindowState := wsNormal;
      visible := true;
      SetForegroundWindow(Application.Handle);
      Shell_NotifyIcon(NIM_Delete, @TrayIcon);
   end
   else
   begin
      Application.Restore;
      SetForeGroundWindow(Application.Handle);
   end;
   Application.BringToFront;

   if forceToForeground then
      ForceForegroundWindow(application.handle);

   feedsFormShowing := currentFeedsFormShowing;
end;


procedure TMainForm.btnNewWebPageClick(Sender: TObject);
begin
   CreateMDIChild;
   feedsFormShowing := false;
end;

function TMainForm.alertUser(formCaption,title,messageText: string; url: string; showRelevancePanel: boolean): TUserResponse;

begin
   bringApplicationToFront;

   with TfrmPageFound.create(self) do
   begin
      setFeedsForm(frmFeeds);
      WindowState := wsNormal;
      setDetails(formCaption, url, title, messageText,showRelevancePanel);
      showmodal;
      result.relevance := getRelevance;
      result.hadEnough := cbEnough.Checked;
      free;
   end;
end;


procedure TMainForm.RSSfeeds1Click(Sender: TObject);
begin
   frmFeeds.WindowState := wsNormal;
   frmFeeds.show;
   feedsFormShowing := true;
end;


{ *** GENERIC FUNCTIONS SHARED WITH ALL UNITS *** }
procedure validateKeywords(var txtKeywords: TMemo);
var
   i: integer;
   corrected: boolean;
begin
   // validate & correct
   corrected := false;
   for i := 0 to txtKeywords.lines.Count-1 do
   begin
      if length(txtKeywords.lines[i]) < 2 then continue;
      if (copy(txtKeywords.lines[i], 1, 2) <> '0:') and (copy(txtKeywords.lines[i], 1, 2) <> '1:') then
      begin
         txtKeywords.lines[i] := '1:' + txtKeywords.lines[i];
         corrected := true;
      end;
   end;
   if corrected then showmessage('Each keyword line must begin with 1: or 0: to indicate whether active or not.  I have fixed this for you.');
end;

procedure processMessagesAtInterval;
begin
   // calls application.processmessages every N seconds
   // this means the screen will be refreshed properly when doing
   // a large download
   if currentTimeMillis - processMessagesInterval > lastProcessMessages then
   begin
      lastProcessMessages := currentTimeMillis;
      application.ProcessMessages;
   end;
end;

end.
